@@ -32,7 +32,7 @@ function(x, ...) {
   })
   x <- structure(list(
     sec=ntime$sec, min=ntime$min, hour=ntime$hour, 
-    mday=ntime$mday, mon=ntime$mon, year=ntime$year,
+    mday=ntime$mday, month=ntime$mon, year=ntime$year,
     wday=ntime$wday, yday=ntime$yday,isdst=ntime$isdst), tzone=attr(x,"tzone"),
     class=c("POSIXlt","POSIXt"))
   do.call(paste('as',oClass[1],sep='.'), list(x))
