@@ -43,11 +43,15 @@ function(x,n=1,keep=FALSE,...)
       xx <- x[sub]
       if(keep) xx <- structure(xx,keep=x[(-(-n)+1):NROW(x)])
       xx
-    } else {
+    } else if(n < 0) {
       sub <- seq.int(to = length(x), length.out = max(length(x)-(-n), 0L))
       xx <- x[sub]
       if(keep) xx <- structure(xx,keep=x[1:(-n)])
       xx
+    } else {
+      xx <- x[0]
+      if(keep) xx <- structure(xx,keep=x[0])
+      xx
     }
   } else {
     if(n > 0) {
@@ -55,11 +59,15 @@ function(x,n=1,keep=FALSE,...)
       xx <- x[sub,,drop=FALSE]
       if(keep) xx <- structure(xx,keep=x[(-(-n)+1):NROW(x),])
       xx
-    } else {
+    } else if(n < 0) {
       sub <- seq.int(to = NROW(x), length.out = max(NROW(x)-(-n), 0L))
       xx <- x[sub,,drop=FALSE]
       if(keep) xx <- structure(xx,keep=x[1:(-n),])
       xx
+    } else {
+      xx <- x[0,,drop=FALSE]
+      if(keep) xx <- structure(xx,keep=x[0,])
+      xx
     }
   }
 }
@@ -77,9 +85,10 @@ function(x,n=1,keep=FALSE,...)
     # series periodicity
     sp <- periodicity(x)
     # requested periodicity$units
+    sp.units <- sp[["units"]]
     rpu <- np[length(np)]
     rpf <- ifelse(length(np) > 1, as.numeric(np[1]), 1)
-    if(rpu == sp$unit) {
+    if(rpu == sp.units) {
       n <- rpf
     } else {
       # if singular - add an s to make it work
@@ -92,7 +101,7 @@ function(x,n=1,keep=FALSE,...)
       if(!rpu %in% dt.options)
         stop(paste("n must be numeric or use",paste(dt.options,collapse=',')))
       dt <- dt.options[pmatch(rpu,dt.options)]
-      if(u.list[[dt]] > u.list[[sp$unit]]) {
+      if(u.list[[dt]] > u.list[[sp.units]]) {
         #  req is for higher freq data period e.g. 100 mins of daily data
         stop(paste("At present, without some sort of magic, it isn't possible",
              "to resolve",rpu,"from",sp$scale,"data"))
@@ -111,7 +120,7 @@ function(x,n=1,keep=FALSE,...)
         }
         if(keep) xx <- structure(xx,keep=x[(ep[-(-rpf)+1]+1):NROW(x)])
         return(xx)
-      } else {
+      } else if(rpf < 0) {
         n <- ep[-rpf+1]+1
         if(is.null(dim(x))) {
           xx <- x[n:NROW(x)]
@@ -120,6 +129,14 @@ function(x,n=1,keep=FALSE,...)
         }
         if(keep) xx <- structure(xx,keep=x[1:(ep[-rpf+1])])
         return(xx)
+      } else {
+        if(is.null(dim(x))) {
+          xx <- x[0]
+        } else {
+          xx <- x[0,,drop=FALSE]
+        }
+        if(keep) xx <- structure(xx,keep=x[0])
+        return(xx)
       }
     }
   }
@@ -133,7 +150,7 @@ function(x,n=1,keep=FALSE,...)
     }
     if(keep) xx <- structure(xx,keep=x[(-(-n)+1):NROW(x)])
     xx
-  } else {
+  } else if(n < 0) {
     if(abs(n) >= NROW(x))
       return(x[0])
     if(is.null(dim(x))) {
@@ -143,5 +160,13 @@ function(x,n=1,keep=FALSE,...)
     }
     if(keep) xx <- structure(xx,keep=x[1:(-n)])
     xx
+  } else {
+    if(is.null(dim(x))) {
+      xx <- x[0]
+    } else {
+      xx <- x[0,,drop=FALSE]
+    }
+    if(keep) xx <- structure(xx,keep=x[0])
+    xx
   }
 }
