@@ -91,7 +91,7 @@ function(x, ...) {
   attr(attr(x, 'index'), 'tclass') <- class(value)
 
   # set tzone attribute
-  if(any(class(value) %in% .classesWithoutTZ)) {
+  if(isClassWithoutTZ(object = value)) {
     attr(attr(x, 'index'), 'tzone') <- 'UTC'
   } else {
     if (is.null(attr(value, 'tzone'))) {
@@ -166,3 +166,11 @@ function(x, ...) {
   ((.index(x) + (3 * 86400)) %/% 86400 %/% 7) -
     ((startOfYear() * 86400 + (3 * 86400)) %/% 86400 %/% 7)[.indexyear(x) + 1]
 }
+
+.update_index_attributes <- function(x) {
+  suppressWarnings({
+    tclass(x) <- tclass(x)
+    tzone(x) <- tzone(x)
+  })
+  return(x)
+}
