@@ -69,13 +69,16 @@ merge.xts <- function(...,
   if(length(suffixes) != length(dots)) {
     warning("length of suffixes and does not match number of merged objects")
     syms <- as.character(dots)
+    # should we ignore suffixes here?
+    #suffixes <- NULL
   } else {
     syms <- as.character(suffixes)
-    sfx  <- as.character(suffixes)
   }
 
-  .times <- .External('number_of_cols', ..., PACKAGE="xts")
-  symnames <- rep(syms, .times)  # moved call to make.names inside of mergeXts/do_merge_xts
+  .times <- .External(C_number_of_cols, ...)
+  # moved call to make.names inside of mergeXts/do_merge_xts
+  symnames <- rep(syms, .times)
+  suffixes <- rep(suffixes, .times)
 
   if(length(dots) == 1) {
     # this is for compat with zoo; one object AND a name
@@ -117,7 +120,7 @@ merge.xts <- function(...,
   if( length(retside) != 2 ) 
     retside <- rep(retside[1], 2)
 
-  x <- .External('mergeXts',
+  x <- .External(C_mergeXts,
             all=all[1:2],
             fill=fill,
             setclass=setclass,
@@ -127,7 +130,7 @@ merge.xts <- function(...,
             env=new.env(),
             tzone=tzone,
             check.names=check.names,
-            ..., PACKAGE="xts")
+            ...)
   if(!is.logical(retclass) && retclass != 'xts') {
     asFun <- paste("as", retclass, sep=".")
     if(!exists(asFun)) {
